/***
 * Fractal ADL Parser
 * Copyright (C) 2002 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl;

import java.util.HashMap;

/**
 * A {@link DelegatingLoader DelegatingLoader} that maintains a cache of already
 * loaded definitions.
 */

public abstract class CachingLoader extends DelegatingLoader {

  /**
   * A cache of already loaded type and template definitions.
   */

  private HashMap cache = new HashMap();

  // -------------------------------------------------------------------------
  // Implementation of the Loader interface
  // -------------------------------------------------------------------------

  public XMLElement loadTypeDescriptor (final String name)
    throws ParserException
  {
    XMLElement desc = (XMLElement)cache.get(name);
    if (desc == null) {
      desc = delegate.loadTypeDescriptor(name);
      try {
        desc = handleTypeDescriptor(name, desc);
      } catch (ParserException e) {
        throw new ParserException(desc, e);
      }
      cache.put(name, desc);
    }
    return desc;
  }

  public XMLElement loadTemplateDescriptor (final String name)
    throws ParserException
  {
    XMLElement desc = (XMLElement)cache.get(name);
    if (desc == null) {
      desc = delegate.loadTemplateDescriptor(name);
      try {
        desc = handleTemplateDescriptor(name, desc);
      } catch (ParserException e) {
        throw new ParserException(desc, e);
      }
      cache.put(name, desc);
    }
    return desc;
  }
}
