/***
 * Julia: France Telecom's implementation of the Fractal API
 * Copyright (C) 2001-2002 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package rmi.clt;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;

import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.rmi.registry.Registry;

import org.objectweb.fractal.util.Fractal;

public class ClientLauncher {

  public static void main (final String[] args) throws Exception {
    // the RMI class loader does not work if there is no security manager
    System.setSecurityManager(new SecurityManager());

    boolean useWrapper = false;
    for (int i = 0; i < args.length; ++i) {
      useWrapper |= args[i].equals("wrapper");
    }

    NamingService ns = Registry.getRegistry();
    Component boot = Fractal.getBootstrapComponent();
    Component rboot = ns.lookup("server-host");

    // constructs the templates manually
    TypeFactory tf = Fractal.getTypeFactory(boot);
    // type of root component
    ComponentType rType = tf.createFcType(new InterfaceType[] {
      tf.createFcItfType("m", "rmi.clt.Main", false, false, false)
    });
    // type of client component
    ComponentType cType = tf.createFcType(new InterfaceType[] {
      tf.createFcItfType("m", "rmi.clt.Main", false, false, false),
      tf.createFcItfType("s", "rmi.clt.Service", true, false, false)
    });
    // type of server component
    ComponentType sType = tf.createFcType(new InterfaceType[] {
      tf.createFcItfType("s", "rmi.clt.Service", false, false, false),
      tf.createFcItfType(
        "attribute-controller",
        "rmi.clt.ServiceAttributes",
        false,
        false,
        false)
    });

    GenericFactory cf = Fractal.getGenericFactory(boot);
    GenericFactory rcf = Fractal.getGenericFactory(rboot);

    // create root component
    Component rComp = cf.newFcInstance(rType, "composite", null);
    // create client component
    Component cComp = cf.newFcInstance(
        cType, "primitive", "rmi.clt.ClientImpl");
    // create server component
    Component somp = rcf.newFcInstance(
        sType, "parametricPrimitive", "rmi.clt.ServerImpl");
    ((ServiceAttributes)Fractal.getAttributeController(somp)).setHeader("-> ");
    ((ServiceAttributes)Fractal.getAttributeController(somp)).setCount(1);

    if (useWrapper) {
      // create client component "wrapper" component
      Component CTmpl = cf.newFcInstance(cType, "composite", null);
      // create server component "wrapper" component
      Component STmpl = rcf.newFcInstance(sType, "composite", null);
      // component assembly
      Fractal.getContentController(CTmpl).addFcSubComponent(cComp);
      Fractal.getContentController(STmpl).addFcSubComponent(somp);
      Fractal.getBindingController(CTmpl).bindFc("m", cComp.getFcInterface("m"));
      Fractal.getBindingController(cComp).bindFc(
        "s", Fractal.getContentController(CTmpl).getFcInternalInterface("s"));
      Fractal.getBindingController(STmpl).bindFc("s", somp.getFcInterface("s"));
      // replaces client and server components by "wrapper" components
      cComp = CTmpl;
      somp = STmpl;
    }

    // component assembly
    Fractal.getContentController(rComp).addFcSubComponent(cComp);
    Fractal.getContentController(rComp).addFcSubComponent(somp);
    Fractal.getBindingController(rComp).bindFc("m", cComp.getFcInterface("m"));
    Fractal.getBindingController(cComp).bindFc("s", somp.getFcInterface("s"));

    // start root component
    Fractal.getLifeCycleController(rComp).startFc();

    // call main method
    ((Main)rComp.getFcInterface("m")).main(new String[0]);
  }
}
